﻿// Accord Math Library
// The Accord.NET Framework
// http://accord-framework.net
//
// Copyright © César Souza, 2009-2013
// cesarsouza at gmail.com
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

namespace Accord.Neuro
{
    using System.Runtime.CompilerServices;
    using Accord.Neuro.Networks;
    using Accord.Neuro.Learning;

    /// <summary>
    ///  Neural learning algorithms such as Levenberg-Marquardt, Parallel Resilient
    ///  Backpropagation, initialization procedures such as Nguyen-Widrow and Deep
    ///  Learning models, such as Restricted Boltzmann Machines and Deep Belief
    ///  Networks.
    /// </summary>
    /// 
    /// <remarks>
    /// <para>
    ///   This namespace has a special focus on Neural Networks and related techniques. 
    ///   It contains several learning algorithms, such as the <see cref="LevenbergMarquardtLearning">
    ///   Levenberg-Marquardt (LM) with Bayesian Regularization</see>, and the much faster
    ///   <see cref="ParallelResilientBackpropagationLearning">Parallel Resilient 
    ///   Backpropagation</see> algorithm. One of the newest additions to the framework
    ///   also include the <see cref="DeepBeliefNetwork">Deep-Belief Network</see> and
    ///   <see cref="RestrictedBoltzmannMachine">Restricted Boltzmann Machines</see> from
    ///   the deep learning paradigm.
    ///  </para>
    ///   
    /// <para>
    ///   The namespace class diagram is shown below. </para>
    ///   <img src="..\diagrams\classes\Accord.Neuro.png" />
    ///   
    /// <para>
    ///   Please note that class diagrams for each of the inner namespaces are 
    ///   also available within their own documentation pages.</para>
    /// </remarks>
    /// 
    /// <seealso cref="Accord.Neuro.ActivationFunctions"/>
    /// <seealso cref="Accord.Neuro.Layers"/>
    /// <seealso cref="Accord.Neuro.Learning"/>
    /// <seealso cref="Accord.Neuro.Networks"/>
    /// <seealso cref="Accord.Neuro.Neurons"/>
    /// <seealso cref="Accord.Neuro.Visualization"/>
    /// 
    [CompilerGenerated]
    class NamespaceDoc
    {
    }
}
